import fs from 'node:fs';
import { Buffer } from 'node:buffer';
import process from 'node:process';
import { createUnplugin } from 'unplugin';
import { createFilter } from '@rollup/pluginutils';

function check(code, options) {
    // check if fold is valid
    for (const [start, end] of options) {
        if (start >= end) {
            return {
                valid: false,
                message: `Invalid fold input: start ${start} >= end ${end}`
            };
        }
    }
    // check if fold is in the code
    const lines = code.split('\n');
    for (const [start, end] of options) {
        if (start >= lines.length || end >= lines.length) {
            return {
                valid: false,
                message: `Invalid fold input: start ${start} or end ${end} >= lines.length ${lines.length}`
            };
        }
    }
    return {
        valid: true,
        message: ''
    };
}

function processFold(code, options) {
    if (options.length === 0) {
        return code;
    }
    const { valid, message } = check(code, options);
    if (!valid) {
        throw new Error(message);
    }
    options.forEach(([start, end]) => {
        // line start end
        const lines = code.split('\n');
        // insert // ...
        lines.splice(start, end - start + 1, '// ...');
        const foldedCode = lines.join('\n');
        code = foldedCode;
    });
    return code;
}

function processTruncate(code, options) {
    if (options.length === 0) {
        return code;
    }
    const { valid, message } = check(code, options);
    if (!valid) {
        throw new Error(message);
    }
    // sort by start
    options.sort((a, b) => a[0] - b[0]);
    const lines = code.split('\n');
    let raw = '';
    for (const [start, end] of options) {
        raw += lines.slice(start, end + 1).join('\n');
    }
    raw += '\n';
    return raw;
}

const defaultInclude = [
    '**/*.vue',
    '**/*.jsx',
    '**/*.tsx',
    '**/*.astro',
    '**/*.svelte',
    '**/*.html',
];
const unpluginFactory = (options = {}) => {
    return {
        name: 'vite-plugin-code-sample',
        transformInclude(id) {
            return filter(options)(id);
        },
        transform(code, id) {
            return transform(code, id, options);
        }
    };
};
function filter(options) {
    return createFilter(options.include || defaultInclude, options.exclude);
}
function transform(code, id, options) {
    var _a;
    if (/<code-sample/.test(code)) {
        const removeInProd = (_a = options === null || options === void 0 ? void 0 : options.removeInProd) !== null && _a !== void 0 ? _a : true;
        if (removeInProd && process.env.NODE_ENV === 'production') {
            // remove all code-sample tags in production
            return code.replace(/<code-sample[^>]*>[\s\S]*?<\/code-sample>/g, '');
        }
        const fileContent = fs.readFileSync(id, 'utf-8');
        // check if <code-sample> has fold and truncate attribute
        const foldOptRaw = code.match(/fold="([^"]+)"/);
        const truncateOptRaw = code.match(/truncate="([^"]+)"/);
        // parse fold and truncate
        const foldOptions = foldOptRaw ? JSON.parse(foldOptRaw[1]) : [];
        const truncateOptions = truncateOptRaw ? JSON.parse(truncateOptRaw[1]) : [];
        // process fold and truncate
        let processedCode = processFold(fileContent, foldOptions);
        processedCode = processTruncate(processedCode, truncateOptions);
        const base64Content = Buffer.from(processedCode).toString('base64');
        return code.replace(/<code-sample/g, `<code-sample data-sample-code="${base64Content}"`);
    }
    return code;
}
const unplugin = /* #__PURE__ */ createUnplugin(unpluginFactory);

export { unplugin as default, filter, transform, unplugin, unpluginFactory };
